% function [V,l]=DiscLap(level,creal,cimag,meas,plotmth,toplot,writeme) 
% ------------------------------------------
% Computes the spectrum of our discrete approximation to a Laplacian on the
% specified quasicircle Julia Set.
% 
% level: level of graph approximation 
% cvalue=creal+i*cimag: specifies c-value should be in the main cardioid
% meas: specifies the measure 
%    -'c' conformal 
%    -'e' equilibrium (default) 
% plotmth: specifies the output method
%    -'r' plots rays 
%    -'s' plots rays and supports  (default)
%    -'h' plots eigenfunctions as a height 
%    -'a' plots both rays and heights 
% maxpics: is the maximum number of eigenfunctions to plot
% writeme: 1 save eigenvalues 
%
% Example [V,l]=DiscLap(5,.2,0,'c','wr',45) will evalute the eigenfunctions
% of the 5th level approximation to the julia set for c=.2 using the
% conformal measure, save the eigenfunctions in 'Qcl5.out' and plot the
% ray angle pictures of the first 45 eigenfunctions 

function [V,l,L,D]=DiscLap(level,creal,cimag,meas,plotmth,toplot,writeme)
% We compute the bilinear forms for energy and measure for our graph
% approximation (see LapQCW for details on approximation), we then create
% the matrix approximation to the Laplacian for the specified measure and
% evalutate it's spectra.

% uses LapQCW.m 

% Taryn Flock Summer 2008

% Handle input variables 
if nargin<3 
    error('Please specify level,creal, and cimag') 
elseif nargin==3
    meas='e';
    fprintf('Running with Equilibrium Measure\n')
    plotmth='s';
    toplot=[1:8];
elseif nargin==4
    plotmth='r';
    maxpics=36;
elseif nargin==5
    maxpics=36;
end

r=2; 

% Generate set up info
fprintf('Creating graph approximation\r')
tic
[rays,vals,weights]=LapQCW(level,creal,cimag);
numpoints=length(rays);
toc 

if (meas=='e')
    weights=ones(1,numpoints )/numpoints;
elseif (meas~='c') 
    error('Please specify a valid measure, either c, conformal, or e, equalibrium')
end 

fprintf('Creating Laplacian Matrix\r')
tic
% Calculate the differences btw ray angles
Drays=[rays,0]-[rays(numpoints)-2^level,rays];
Drays=Drays(1:numpoints);

% Calculate Energy Matrix
E1=diag(-1./Drays,0)+diag(1./Drays(2:numpoints),1);
E1(numpoints,1)=1/Drays(1);   % difference quotient in u

E2=diag(-1*ones(1,numpoints),0)+diag(ones(1,numpoints-1),-1);
E2(1,numpoints)=1; % difference quotient in v * dtheta (cancels denom) 

E=r^(level)*E1*E2; % true energy matrix

% Calculate 'Integral' wrt measure matrix
G=diag(weights); 

% Laplacian Matrix
 L=inv(G)*E;
toc 
% Calculate and order eigenvalues
fprintf('Finding Spectral Information\r')
tic 
[V,D]=eig(L,'nobalance');
[l,I]=sort(nonzeros(D));
V=V(:,I);
toc 

% Output Data 
% -----------------------------------------------------------------

fprintf('Outputing Results\r') 

if writeme==1; 
    filename=strcat('Qc',num2str(creal),'i',num2str(cimag),meas,num2str(level),'.out');
    fid=fopen(filename,'w');
    if (fid==-1); error('Bad Filename'); end
    fprintf(fid,'% f\n', l);
    fclose(fid);
end 


% Make Pretty Pictures of the eigenfunctions
MaxPics=length(toplot); 
if plotmth=='s' 
    for j=1:ceil(MaxPics/4)
        figure;
        for i=1:min(4,MaxPics-4*(j-1))
            eignum = toplot(i+4*(j-1));
            subplot(4,2,2*i-1);
            % Plot Rays 
            eigfun=V(:,eignum);
            plot(rays/(max(rays)+1),eigfun,'LineWidth',1);
            title(strcat('Num',num2str(eignum)))
            axis square
            axis tight
            
            subplot(4,2,2*i);
            % Look at sign of sufficiently nonzero points 
            goodpt=abs(eigfun)>(max(V(:,1))-min(V(:,1)));
            goodpt=goodpt&(abs(eigfun)>(max(abs(eigfun))/10000));
            if(eignum==64) 
                sum(goodpt)
            end
            signpt=sign(eigfun(goodpt));
            scatter(real(vals(goodpt)),imag(vals(goodpt)),5*abs(signpt),signpt, 'filled')
            title(strcat(' Eigval ',num2str(l(eignum))))
            colormap('winter');
            axis tight
            axis square     
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
    if writeme==1 
        filename = strcat('Q',num2str(level),meas,'Pic',num2str(j),'.png'); 
        print('-dpng', filename)
        close
    end
    end
elseif plotmth=='r' 
      for j=1:ceil(MaxPics/3)
        figure;
        for i=1:min(3,MaxPics-3*(j-1))
            eignum = toplot(i+3*(j-1));
            subplot(3,1,i);
            % Plot Rays 
            eigfun=V(:,eignum);
            plot(rays/(max(rays)+1),eigfun,'LineWidth',1);
            title(strcat('Num ',num2str(eignum),' EigVal ',num2str(l(eignum))))
            axis tight
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
        if writeme==1 
            filename = strcat('B',num2str(level),meas,'Pic',num2str(j),'.png'); 
            print('-dpng', filename)
            close
        end
      end
elseif plotmth=='h'      
        for i=toplot;
            figure;
            scatter3(real(vals),imag(vals),M*V(:,i),10,6*M*V(:,i),'filled');
        end
elseif plotmth=='b'
  for i=toplot
      figure;
      scatter3(real(vals),imag(vals),M*V(:,i),15,M*V(:,i),'filled');
      axis([-2 2 -1.5 1.5 min(V(:,i)), max(V(:,i))]) 
  end
elseif plotmth~='n' 
    fprintf('Invalid plot method please specify r,s or n\r')
end

        
end

